#!/bin/bash

hfs()
{
# Create one stat file for each value in BURSTS, containing the
# average execution time for each scheduler in SCHEDS and for each
# flowset in FLOWSETS
local MAC="qfq+ rr wf2qp fifo w2f2q"
local MACSTAT=(QFQ+ DRR WF^\{2\}Q+ FIFO W^\{2\}F^\{2\}Q )
#MAC="qfq+"
#QMAC="1 10 20 40 60 80 100 120 140 160 180 200 220 240 260 280 300"
local QMAC="1 10 20 40 60 80 100 120 140 160 180 200 220 240 250"
local FLOWMAC="3"  #"1 2 0" if we want also constant weight distribution
local CMD=./test
local FIELDNR=8
local NUM_ITER=1
local NUM_EVENTS=1m
local FLOWSETS_NAMES=( analogical linear  constant )

local FILENAME=output_description.out

local qmin=0
local qmax=300
local BURST=1
local QSING=0

echo "# output description:"  > $FILENAME
echo "# 1:Mac_buff_size  2:Throughput  3:Delay_10-1  4:Delay_10-7  
5:avg_delay_MAC 6:avg_delay_MAC_time  7:RFi(Relative)  8:WFi    
9:Queueing_delay_time_QoS  10:Queueing_delay_10-7_mac 
11:queueing_delay_10-2_mac   12:exec_time" s>> $FILENAME
echo >> $FILENAME

  counter=0
  for flowmac in $FLOWMAC
  do
    counter_sched=-1
    for mac in $MAC
    do
      counter_sched=$(($counter_sched + 1))
      echo >> test_${FLOWSETS_NAMES[$counter]}.out #white space
      echo >> test_${FLOWSETS_NAMES[$counter]}.out #white space
      echo >> $FILENAME
      echo ${MACSTAT[$counter_sched]} >> test_${FLOWSETS_NAMES[$counter]}.out
      echo $mac >> $FILENAME
      for qmac in $QMAC
      do
        echo Scheduler $mac, Flow ${FLOWSETS_NAMES[$counter]}, Q $qmac		
				
        thr=0
        del_one=0
        del_seven=0
        avg_del=0
        avg_time=0
        fair=0
        wfi=0
        qdel=0
        qdelMac_seven=0
        qdelMac_two=0
        for ((i = 0 ; $i < $NUM_ITER; i++))
        do
          if [ "$mac" == "w2f2q" ]
          then
            printf "Iteration $((i + 1))/$NUM_ITER\n"
            stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
                  20::20,10::20,1::60 -burst $BURST -qsing $QSING  -stats 2 \
                  -mode double -qos qfq+ -mac wf2qp -qmac $qmac -flowsmac 20 \
                  -wdistr $flowmac -intgr_th 50 ) #>> $FILENAME
          else
            printf "Iteration $((i + 1))/$NUM_ITER\n"
            stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
                  20::20,10::20,1::60 -burst $BURST -qsing $QSING  -stats 2 \
                  -mode double -qos qfq+ -mac $mac -qmac $qmac -flowsmac 20 \
                  -wdistr $flowmac ) #>> $FILENAME
          fi
          echo $stats >> $FILENAME
          counter_stats=0
          for sts in $stats
          do
            if [ $counter_stats -eq 1 ]
            then
              thr=$(echo $thr+$sts | bc)
            fi
            if [ $counter_stats -eq 2 ]
            then
              del_one=$(echo $del_one+$sts | bc)
            fi
            if [ $counter_stats -eq 3 ]
            then
              del_seven=$(echo $del_seven+$sts | bc)
            fi
            if [ $counter_stats -eq 4 ]
            then
              avg_del=$(echo $avg_del+$sts | bc)
            fi
            if [ $counter_stats -eq 5 ]
            then
              fair=$(echo $fair+$sts | bc)
            fi
            if [ $counter_stats -eq 6 ]
            then
              wfi=$(echo $wfi+$sts | bc)
            fi
            if [ $counter_stats -eq 7 ]
            then
              qdel=$(echo $qdel+$sts | bc)
            fi
            if [ $counter_stats -eq 8 ]
            then
              qdelMac_seven=$(echo $qdelMac_seven+$sts | bc)
            fi
            if [ $counter_stats -eq 9 ]
            then
              qdelMac_two=$(echo $qdelMac_two+$sts | bc)
            fi
            if [ $counter_stats -eq 10 ]
            then
              avg_time=$(echo $avg_time+$sts | bc)
            fi
          counter_stats=$(($counter_stats + 1))
          done

        done
      thr=$(echo $thr/$NUM_ITER | bc -l)
      del_one=$(echo $del_one/$NUM_ITER | bc -l)
      del_seven=$(echo $del_seven/$NUM_ITER | bc -l)
      avg_del=$(echo $avg_del/$NUM_ITER | bc -l)
      fair=$(echo $fair/$NUM_ITER | bc -l)
      wfi=$(echo $wfi/$NUM_ITER | bc -l)
      qdel=$(echo $qdel/$NUM_ITER | bc -l)
      qdelMac_seven=$(echo $qdelMac_seven/$NUM_ITER | bc -l)
      qdelMac_two=$(echo $qdelMac_two/$NUM_ITER | bc -l)
      avg_time=$(echo $avg_time/$NUM_ITER | bc -l)
    
      echo $qmac  $thr  $del_one   $del_seven  $avg_del   $fair \
           $wfi   $qdel   $qdelMac_seven   $qdelMac_two \
           $avg_time >> test_${FLOWSETS_NAMES[$counter]}.out
      echo $CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
           20::20,10::20,1::60 -burst $BURST -qsing $QSING  -stats 2 -mode \
           double -qos qfq+ -mac $mac -qmac $qmac -flowsmac 20 -wdistr $flowmac
      done
    echo >> $FILENAME 
    done
  counter=$(($counter + 1))
  done

mv *.out ../graphs/output_test/

  sleep 1

#tests on double schedulers
SCHED="hfs doublerr wf2qp rr" # "double1 double2"
SCHEDSTAT=( HFS double-DRR single-WF^\{2\}Q+ single-DRR )

FILENAME_TOTAL=total_output_descrition.out

echo "# output description:"  > $FILENAME_TOTAL
echo "# 1:Mac_buff_size  2:QueueingDelay_total  3:exec_time" s>> $FILENAME_TOTAL
echo >> $FILENAME_TOTAL

  counter=0
  for flowmac in $FLOWMAC
  do
    counter_sched=-1
    for sched in $SCHED
    do
      counter_sched=$(($counter_sched + 1))
      echo >> total_test_${FLOWSETS_NAMES[$counter]}.out #white space
      echo >> total_test_${FLOWSETS_NAMES[$counter]}.out #white space
      echo >> $FILENAME_TOTAL
      echo ${SCHEDSTAT[$counter_sched]} >> \
           total_test_${FLOWSETS_NAMES[$counter]}.out
      echo $sched >> $FILENAME_TOTAL
      for qmac in $QMAC
      do
        echo Scheduler $sched, Flow ${FLOWSETS_NAMES[$counter]}, Q $qmac		
				
        qdel=0
        wfi=0
        wfi_interm=0
        twfi=0
        twfi_interm=0
        avg_time=0
        for ((i = 0 ; $i < $NUM_ITER; i++))
        do
          if [ "$sched" == "hfs" ]
          then
            printf "Iteration $((i + 1))/$NUM_ITER\n"
            stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
                  20::20,10::20,1::60 -burst $BURST -qsing $QSING -stats 4 \
                  -mode double -qos qfq+ -mac qfq+ -qmac $qmac -flowsmac 20 \
                  -wdistr $flowmac ) #>> $FILENAME_TOTAL
          else
            if [ "$sched" == "doublerr" ]
            then
              printf "Iteration $((i + 1))/$NUM_ITER\n"
              stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
                    20::20,10::20,1::60 -burst $BURST -qsing $QSING -stats 4 \
                    -mode double -qos rr -mac rr -qmac $qmac -flowsmac 20 \
                    -wdistr $flowmac ) #>> $FILENAME_TOTAL
            else
              if [ "$sched" == "double1" ]
              then
                printf "Iteration $((i + 1))/$NUM_ITER\n"
                stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
                      20::20,10::20,1::60 -burst $BURST -qsing $QSING -stats 4 \
                      -mode double -qos wf2qp -mac wf2qpp -qmac $qmac \
                      -flowsmac 20 -wdistr $flowmac ) #>> $FILENAME_TOTAL
              else
                if [ "$sched" == "double2" ]
                then
                  printf "Iteration $((i + 1))/$NUM_ITER\n"
                  stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax \
                        -flowsets 20::20,10::20,1::60 -burst $BURST -qsing \
                        $QSING -stats 4 -mode double -qos qfq -mac qfq+ -qmac \
                        $qmac -flowsmac 20 -wdistr $flowmac ) #>> $FILENAME
                else
                  printf "Iteration $((i + 1))/$NUM_ITER\n"
                  stats=$($CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax \
                        -flowsets 20::20,10::20,1::60 -burst $BURST -qsing \
                        $QSING -stats 4 -mode single -alg $sched )
                fi
              fi
            fi
          fi
          echo $stats >> $FILENAME_TOTAL
          counter_stats=0
          for sts in $stats
          do
            if [ $counter_stats -eq 0 ]
            then
              qdel=$(echo $qdel+$sts | bc)
            fi
            if [ $counter_stats -eq 1 ]
            then
              wfi=$(echo $wfi+$sts | bc)
            fi
            if [ $counter_stats -eq 2 ]
            then
              wfi_interm=$(echo $wfi_interm+$sts | bc)
            fi
            if [ $counter_stats -eq 3 ]
            then
              twfi=$(echo $twfi+$sts | bc)
            fi
            if [ $counter_stats -eq 4 ]
            then
              twfi_interm=$(echo $twfi_interm+$sts | bc)
            fi
            if [ $counter_stats -eq 5 ]
            then
              avg_time=$(echo $avg_time+$sts | bc)
            fi
          counter_stats=$(($counter_stats + 1))
          done
        done
      qdel=$(echo $qdel/$NUM_ITER | bc -l)
      wfi=$(echo $wfi/$NUM_ITER | bc -l)
      wfi_interm=$(echo $wfi_interm/$NUM_ITER | bc -l)
      twfi=$(echo $twfi/$NUM_ITER | bc -l)
      twfi_interm=$(echo $twfi_interm/$NUM_ITER | bc -l)
      avg_time=$(echo $avg_time/$NUM_ITER | bc -l)

      echo $qmac  $qdel   $wfi  $wfi_interm  $twfi  $twfi_interm  $avg_time  \
           >> total_test_${FLOWSETS_NAMES[$counter]}.out
      echo $CMD -n $NUM_EVENTS -qmin $qmin -qmax $qmax -flowsets \
           20::20,10::20,1::60 -burst $BURST -qsing $QSING -stats 4 -mode \
           single -alg $sched -qmac $qmac -flowsmac 20 -wdistr $flowmac
      done
    echo >> $FILENAME_TOTAL
    done
  counter=$(($counter + 1))
  done

mv *.out ../graphs/output_test/

  sleep 1
}

qfqplus()
{
# Creates one or two stat files for each value in BURSTS, containing
# statistics for each scheduler in SCHEDS and for each flowset in
# FLOWSETS.
#
# In particular, if the perf option is given, this script creates two
# files, containing, respectively, the number of instructions executed
# and the number of cache-misses caused by the test
# command. Otherwise, it creates one file, containing the average
# among the execution times reported by the test command.

BURSTS="0 2 50"
SCHEDS="qfq+ rr qfq wf2qp fifo"
FLOWSETS="1::1k 1::500,2::250,8::125 1::32k 1::16k,2::8k,8::4k"
NUM_ITER=10
NUM_EVENTS=50M
FLOWSETS_NAMES=( 1k-w1 1k-wmix 32k-w1 32k-wmix )

if [[ "$1" == "perf" ]]; then
    echo Collecting stats on number of instructions and cache misses
    CMD="perf stat -e instructions,cache-misses -x ---- --log-fd 1 ./test"
    TAG=totmisses
    FIELDNR=3 # field containing the number of cache misses
    FIELD_INSTR_NR=1
else
    echo Collecting stats on execution times
    CMD=./test
    TAG=time
    FIELDNR=8 # field containing the execution time reported by test
fi

for burst in $BURSTS
do
    echo Burst $burst
    counter=0
    
    FINAL_FILE_1=test-$TAG-b$burst.out
    echo \# Average $TAG, generated by run_tests.sh, for burst = $burst \
	> $FINAL_FILE_1
    printf "# counter\tflowset\t$SCHEDS\n" >> $FINAL_FILE_1
    if [[ "$1" == "perf" ]]; then
	FINAL_FILE_INSTR=test-totinstr-b$burst.out
	
	echo \# Average total instructions, generated by run_tests.sh, \
	    for burst = $burst > $FINAL_FILE_INSTR
	printf "# counter\tflowset\t$SCHEDS\n" >> $FINAL_FILE_INSTR
    fi

    for fset in $FLOWSETS
    do
	echo Flowset $fset
	printf "$counter\t\t${FLOWSETS_NAMES[$counter]}\t" \
	    >> $FINAL_FILE_1
	if [[ "$1" == "perf" ]]; then
	    printf "$counter\t\t${FLOWSETS_NAMES[$counter]}\t" \
		>> $FINAL_FILE_INSTR
	fi
	echo
	for sched in $SCHEDS
	do
	    echo Scheduler $sched
	    SAMPLE_FILE_1=\
test-$sched-$TAG-${FLOWSETS_NAMES[$counter]}-b$burst.out
	    SAMPLE_FILE_INSTR=\
test-$sched-instr-${FLOWSETS_NAMES[$counter]}-b$burst.out
	    rm -f $SAMPLE_FILE_1
	    for ((i = 0 ; $i < $NUM_ITER; i++))
	    do
		printf "Iteration $((i + 1))/$NUM_ITER"
		CMD_OUT=`$CMD -n $NUM_EVENTS -qmin 0 -qmax 30n -flowsets $fset \
		    -burst $burst -mode single -alg $sched 2> /dev/null`

		if [[ "$1" == "perf" ]]; then
		    CMD_OUT=`$CMD -n $NUM_EVENTS -qmin 0 -qmax 30n \
			-flowsets $fset -burst $burst -mode single \
			-alg $sched 2> /dev/null`
		    
		    CMD_OUT=$(echo $CMD_OUT | sed 's/----/ /g')
		    FIELD_INSTR=$(echo $CMD_OUT | \
			awk "{print \$$FIELD_INSTR_NR}")
		    printf " $FIELD_INSTR "
		    echo $FIELD_INSTR >> $SAMPLE_FILE_INSTR
		else
		    $CMD -n $NUM_EVENTS -qmin 0 -qmax 30n \
			-flowsets $fset -burst $burst -mode single \
			-alg $sched > tmp 2>&1
		    CMD_OUT=`tail -n 1 tmp`
		    rm tmp
		fi
		FIELD=$(echo $CMD_OUT | awk "{print \$$FIELDNR}")
		echo " $FIELD"
		echo $FIELD >> $SAMPLE_FILE_1
	    done
	    if [[ "$1" == "perf" ]]; then
		STATS=$(./calc_avg_and_co.sh 95 < $SAMPLE_FILE_INSTR)
		FIELD=$(echo $STATS | awk "{print \$8}" | sed 's/,/./')
		printf "$FIELD " >> $FINAL_FILE_INSTR
	    fi
	    STATS=$(./calc_avg_and_co.sh 95 < $SAMPLE_FILE_1)
	    FIELD=$(echo $STATS | awk "{print \$8}" | sed 's/,/./')
	    printf "$FIELD " >> $FINAL_FILE_1
	    echo
	    rm -f $SAMPLE_FILE_1 $SAMPLE_FILE_INSTR
	done
	    if [[ "$1" == "perf" ]]; then
		echo | tee -a $FINAL_FILE_INSTR
	    fi
	echo | tee -a $FINAL_FILE_1
	counter=$(($counter + 1))
	sleep 1
    done

    if [[ "$1" == "perf" ]]; then
	FINAL_FILE_INSTR=test-instr-b$burst.out
	echo \# Average instructions, generated by run_tests.sh, \
	    for burst = $burst > $FINAL_FILE_INSTR
	awk '{if (NR == 2) print $0; else if (NR > 2) \
print $1 "\t\t" $2 "\t" $3/25000000 \
" " $4/25000000 " " $5/25000000 " " $6/25000000 " " $7/25000000}' \
< test-totinstr-b$burst.out >> $FINAL_FILE_INSTR
	
	FINAL_FILE_MISSES=test-misses-b$burst.out
	echo \# Average misses, generated by run_tests.sh, \
	    for burst = $burst > $FINAL_FILE_MISSES
	awk '{if (NR == 2) print $0; else if (NR > 2) \
print $1 "\t\t" $2 "\t" $3/25000000 \
" " $4/25000000 " " $5/25000000 " " $6/25000000 " " $7/25000000}' \
< test-totmisses-b$burst.out >> $FINAL_FILE_MISSES

    fi
done

}

usage()
{
    echo -e "Usage:\t$0 <mode> [<perf>]"
    echo "Where mode must be either: qfq+ or hfs."
    echo
    echo In qfq+ mode, the script executes the full batch of experiments reported
    echo in the technical report at
    echo http://www.algogroup.unimore.it/people/paolo/agg-sched/agg-sched.pdf
    echo and in the paper
    echo P. Valente, “Providing Near-Optimal Fair-Queueing Guarantees at
    echo Round-Robin Amortized Cost”, Proceedings of the 22nd International
    echo Conference on Computer Communication and Networks \(ICCCN 2013\), 2013.
    echo
    echo -e "In hfs modes, the script executes tests for the hfs scheduler."
    echo
    echo -n "Only for qfq+ mode, the word perf can be added, optionally, "
    echo "to collect perf statistics."
}

if [[ "$1" == "qfq+" ]]; then
    qfqplus $2
elif [[ "$1" == "hfs" ]]; then
    hfs
else
    usage
fi
